@@ -1,4 +1,3 @@
-/* $Id: browser.c 5677 2016-02-25 14:08:47Z bens $ */
 /**************************************************************************
  *   browser.c                                                            *
  *                                                                        *
@@ -42,136 +41,118 @@ static int longest = 0;
 	/* The number of columns in the longest filename in the list. */
 static size_t selected = 0;
 	/* The currently selected filename in the list; zero-based. */
-static char *path_save = NULL;
-	/* A copy of the current path. */
 
 /* Our main file browser function.  path is the tilde-expanded path we
  * start browsing from. */
-char *do_browser(char *path, DIR *dir)
+char *do_browser(char *path)
 {
     char *retval = NULL;
     int kbinput;
-    bool old_const_update = ISSET(CONST_UPDATE);
-    char *prev_dir = NULL;
-	/* The directory we were in before backing up to "..". */
-    char *ans = NULL;
-	/* The last answer the user typed at the statusbar prompt. */
+    char *present_name = NULL;
+	/* The name of the currently selected file, or of the directory we
+	 * were in before backing up to "..". */
     size_t old_selected;
-	/* The selected file we had before the current selected file. */
+	/* The number of the selected file before the current selected file. */
     functionptrtype func;
 	/* The function of the key the user typed in. */
+    DIR *dir;
+	/* The directory whose contents we are showing. */
 
     /* Don't show a cursor in the file list. */
     curs_set(0);
     blank_statusbar();
-    bottombars(MBROWSER);
-    wnoutrefresh(bottomwin);
 
-    UNSET(CONST_UPDATE);
+  read_directory_contents:
+	/* We come here when we refresh or select a new directory. */
 
-    ans = mallocstrcpy(NULL, "");
+    path = free_and_assign(path, get_full_path(path));
 
-  change_browser_directory:
-	/* We go here after we select a new directory. */
+    if (path != NULL)
+	dir = opendir(path);
 
-    /* Start with no key pressed. */
-    kbinput = ERR;
-
-    path = mallocstrassn(path, get_full_path(path));
-
-    /* Save the current path in order to be used later. */
-    path_save = path;
+    if (path == NULL || dir == NULL) {
+	statusline(ALERT, "Cannot open directory: %s", strerror(errno));
+	/* If we don't have a file list yet, there is nothing to show. */
+	if (filelist == NULL) {
+	    napms(1200);
+	    lastmessage = HUSH;
+	    free(path);
+	    free(present_name);
+	    return NULL;
+	}
+	path = mallocstrcpy(path, present_path);
+	present_name = mallocstrcpy(present_name, filelist[selected]);
+    }
 
     assert(path != NULL && path[strlen(path) - 1] == '/');
 
-    /* Get the file list, and set longest and width in the process. */
-    browser_init(path, dir);
+    if (dir != NULL) {
+	/* Get the file list, and set longest and width in the process. */
+	read_the_list(path, dir);
+	closedir(dir);
+	dir = NULL;
+    }
 
-    assert(filelist != NULL);
+    /* If given, reselect the present_name and then discard it. */
+    if (present_name != NULL) {
+	browser_select_dirname(present_name);
 
-    /* Sort the file list. */
-    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
-
-    /* If prev_dir isn't NULL, select the directory saved in it, and
-     * then blow it away. */
-    if (prev_dir != NULL) {
-	browser_select_dirname(prev_dir);
-
-	free(prev_dir);
-	prev_dir = NULL;
+	free(present_name);
+	present_name = NULL;
     /* Otherwise, select the first file or directory in the list. */
     } else
 	selected = 0;
 
     old_selected = (size_t)-1;
 
+    present_path = mallocstrcpy(present_path, path);
+
     titlebar(path);
 
     while (TRUE) {
-	struct stat st;
-	int i;
-	size_t fileline = selected / width;
-		/* The line number the selected file is on. */
-	char *new_path;
-		/* The path we switch to at the "Go to Directory"
-		 * prompt. */
-
 	/* Make sure that the cursor is off. */
 	curs_set(0);
+	lastmessage = HUSH;
 
-#ifndef NANO_TINY
-	if (kbinput == KEY_WINCH) {
-	    /* Rebuild the file list and sort it. */
-	    browser_init(path_save, opendir(path_save));
-	    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
+	bottombars(MBROWSER);
 
-	    /* Make sure the selected file is within range. */
-	    if (selected >= filelist_len)
-		selected = filelist_len - 1;
-	}
-#endif
-	/* Display (or redisplay) the file list if we don't have a key yet,
-	 * or the list has changed, or the selected file has changed. */
-	if (kbinput == ERR || kbinput == KEY_WINCH || old_selected != selected)
+	/* Display (or redisplay) the file list if the list itself or
+	 * the selected file has changed. */
+	if (old_selected != selected)
 	    browser_refresh();
 
 	old_selected = selected;
 
 	kbinput = get_kbinput(edit);
 
-#ifndef NANO_TINY
-	if (kbinput == KEY_WINCH)
-	    continue;
-#endif
-
 #ifndef DISABLE_MOUSE
 	if (kbinput == KEY_MOUSE) {
 	    int mouse_x, mouse_y;
 
-	    /* We can click on the edit window to select a
-	     * filename. */
+	    /* We can click on the edit window to select a filename. */
 	    if (get_mouseinput(&mouse_x, &mouse_y, TRUE) == 0 &&
-		wmouse_trafo(edit, &mouse_y, &mouse_x, FALSE)) {
+			wmouse_trafo(edit, &mouse_y, &mouse_x, FALSE)) {
 		/* longest is the width of each column.  There
 		 * are two spaces between each column. */
-		selected = (fileline / editwinrows) *
-				(editwinrows * width) + (mouse_y *
-				width) + (mouse_x / (longest + 2));
+		selected = selected - selected % (editwinrows * width) +
+				(mouse_y * width) + (mouse_x / (longest + 2));
 
 		/* If they clicked beyond the end of a row,
 		 * select the last filename in that row. */
 		if (mouse_x > width * (longest + 2))
 		    selected--;
 
-		/* If we're off the screen, select the last filename. */
+		/* If we're beyond the list, select the last filename. */
 		if (selected > filelist_len - 1)
 		    selected = filelist_len - 1;
 
-		/* If we selected the same filename as last time,
-		 * put back the Enter key so that it's read in. */
+		/* If we selected the same filename as last time, fake a
+		 * press of the Enter key so that the file is read in. */
 		if (old_selected == selected)
-		    unget_kbinput(sc_seq_or(do_enter, 0), FALSE, FALSE);
+		    unget_kbinput(sc_seq_or(do_enter, 0), FALSE);
 	    }
+
+	    continue;
 	}
 #endif /* !DISABLE_MOUSE */
 
@@ -179,11 +160,17 @@ char *do_browser(char *path, DIR *dir)
 
 	if (func == total_refresh) {
 	    total_redraw();
+#ifndef NANO_TINY
+	    /* Simulate a window resize to force a directory reread. */
+	    kbinput = KEY_WINCH;
+#endif
 	} else if (func == do_help_void) {
 #ifndef DISABLE_HELP
 	    do_help_void();
+#ifndef NANO_TINY
 	    /* The window dimensions might have changed, so act as if. */
 	    kbinput = KEY_WINCH;
+#endif
 #else
 	    say_there_is_no_help();
 #endif
@@ -193,102 +180,106 @@ char *do_browser(char *path, DIR *dir)
 	} else if (func == do_research) {
 	    /* Search for another filename. */
 	    do_fileresearch();
-	} else if (func == do_page_up) {
-	    if (selected >= (editwinrows + fileline % editwinrows) * width)
-		selected -= (editwinrows + fileline % editwinrows) * width;
-	    else
-		selected = 0;
-	} else if (func == do_page_down) {
-	    selected += (editwinrows - fileline % editwinrows) * width;
-	    if (selected > filelist_len - 1)
+	} else if (func == do_left) {
+	    if (selected > 0)
+		selected--;
+	} else if (func == do_right) {
+	    if (selected < filelist_len - 1)
+		selected++;
+#ifndef NANO_TINY
+	} else if (func == do_prev_word_void) {
+	    selected -= (selected % width);
+	} else if (func == do_next_word_void) {
+	    selected += width - 1 - (selected % width);
+	    if (selected >= filelist_len)
 		selected = filelist_len - 1;
+#endif
+	} else if (func == do_up_void) {
+	    if (selected >= width)
+		selected -= width;
+	} else if (func == do_down_void) {
+	    if (selected + width <= filelist_len - 1)
+		selected += width;
+	} else if (func == do_page_up) {
+	    if (selected < width)
+		selected = 0;
+	    else if (selected < editwinrows * width)
+		selected = selected % width;
+	    else
+		selected -= editwinrows * width;
+	} else if (func == do_page_down) {
+	    if (selected + width >= filelist_len - 1)
+		selected = filelist_len - 1;
+	    else if (selected + editwinrows * width >= filelist_len)
+		selected = (selected + editwinrows * width - filelist_len) %
+				width +	filelist_len - width;
+	    else
+		selected += editwinrows * width;
 	} else if (func == do_first_file) {
 	    selected = 0;
 	} else if (func == do_last_file) {
 	    selected = filelist_len - 1;
 	} else if (func == goto_dir_void) {
-	    /* Go to a specific directory. */
-	    i = do_prompt(TRUE,
+	    /* Ask for the directory to go to. */
+	    int i = do_prompt(TRUE,
 #ifndef DISABLE_TABCOMP
 			FALSE,
 #endif
-			MGOTODIR, ans,
+			MGOTODIR, NULL,
 #ifndef DISABLE_HISTORIES
 			NULL,
 #endif
 			/* TRANSLATORS: This is a prompt. */
 			browser_refresh, _("Go To Directory"));
 
-	    bottombars(MBROWSER);
-
 	    /* If the directory begins with a newline (i.e. an
 	     * encoded null), treat it as though it's blank. */
 	    if (i < 0 || *answer == '\n') {
 		statusbar(_("Cancelled"));
 		continue;
-	    } else if (i != 0) {
-		/* Put back the "Go to Directory" key and save
-		 * answer in ans, so that the file list is displayed
-		 * again, the prompt is displayed again, and what we
-		 * typed before at the prompt is displayed again. */
-		unget_kbinput(sc_seq_or(do_gotolinecolumn_void, 0), FALSE, FALSE);
-		ans = mallocstrcpy(ans, answer);
-		continue;
 	    }
 
 	    /* Convert newlines to nulls in the directory name. */
 	    sunder(answer);
 	    align(&answer);
 
-	    new_path = real_dir_from_tilde(answer);
+	    path = free_and_assign(path, real_dir_from_tilde(answer));
 
-	    if (new_path[0] != '/') {
-		new_path = charealloc(new_path, strlen(path) +
-				strlen(answer) + 1);
-		sprintf(new_path, "%s%s", path, answer);
+	    /* If the given path is relative, join it with the current path. */
+	    if (*path != '/') {
+		path = charealloc(path, strlen(present_path) +
+						strlen(answer) + 1);
+		sprintf(path, "%s%s", present_path, answer);
 	    }
 
 #ifndef DISABLE_OPERATINGDIR
-	    if (check_operating_dir(new_path, FALSE)) {
-		statusbar(_("Can't go outside of %s in restricted mode"),
-				operating_dir);
-		free(new_path);
+	    if (check_operating_dir(path, FALSE)) {
+		/* TRANSLATORS: This refers to the confining effect of the
+		 * option --operatingdir, not of --restricted. */
+		statusline(ALERT, _("Can't go outside of %s"),
+				full_operating_dir);
+		path = mallocstrcpy(path, present_path);
 		continue;
 	    }
 #endif
+	    /* Snip any trailing slashes, so the name can be compared. */
+	    while (strlen(path) > 1 && path[strlen(path) - 1] == '/')
+		path[strlen(path) - 1] = '\0';
 
-	    dir = opendir(new_path);
-	    if (dir == NULL) {
-		/* We can't open this directory for some reason.
-		* Complain. */
-		statusbar(_("Error reading %s: %s"), answer,
-				strerror(errno));
-		beep();
-		free(new_path);
-		continue;
-	    }
+	    /* In case the specified directory cannot be entered, select it
+	     * (if it is in the current list) so it will be highlighted. */
+	    for (i = 0; i < filelist_len; i++)
+		if (strcmp(filelist[i], path) == 0)
+		    selected = i;
 
-	    /* Start over again with the new path value. */
-	    free(path);
-	    path = new_path;
-	    goto change_browser_directory;
-	} else if (func == do_up_void) {
-	    if (selected >= width)
-		selected -= width;
-	} else if (func == do_down_void) {
-	    if (selected + width <= filelist_len - 1)
-		selected += width;
-	} else if (func == do_left) {
-	    if (selected > 0)
-		selected--;
-	} else if (func == do_right) {
-	    if (selected < filelist_len - 1)
-		selected++;
+	    /* Try opening and reading the specified directory. */
+	    goto read_directory_contents;
 	} else if (func == do_enter) {
-	    /* We can't move up from "/". */
+	    struct stat st;
+
+	    /* It isn't possible to move up from the root directory. */
 	    if (strcmp(filelist[selected], "/..") == 0) {
-		statusbar(_("Can't move up a directory"));
-		beep();
+		statusline(ALERT, _("Can't move up a directory"));
 		continue;
 	    }
 
@@ -297,59 +288,57 @@ char *do_browser(char *path, DIR *dir)
 	     * directory if it's ".." or if it's a symlink to a
 	     * directory outside the operating directory. */
 	    if (check_operating_dir(filelist[selected], FALSE)) {
-		statusbar(_("Can't go outside of %s in restricted mode"),
-				operating_dir);
-		beep();
+		statusline(ALERT, _("Can't go outside of %s"),
+				full_operating_dir);
 		continue;
 	    }
 #endif
-
+	    /* If for some reason the file is inaccessible, complain. */
 	    if (stat(filelist[selected], &st) == -1) {
-		/* We can't open this file for some reason.
-		 * Complain. */
-		 statusbar(_("Error reading %s: %s"),
+		statusline(ALERT, _("Error reading %s: %s"),
 				filelist[selected], strerror(errno));
-		 beep();
-		 continue;
+		continue;
 	    }
 
+	    /* If it isn't a directory, a file was selected -- we're done. */
 	    if (!S_ISDIR(st.st_mode)) {
-		/* We've successfully opened a file, we're done, so
-		 * get out. */
 		retval = mallocstrcpy(NULL, filelist[selected]);
 		break;
 	    }
 
-	    dir = opendir(filelist[selected]);
-
-	    if (dir == NULL) {
-		statusbar(_("Error reading %s: %s"),
-				filelist[selected], strerror(errno));
-		beep();
-		continue;
-	    }
-
-	    /* If we moved up one level, remember where we came from, so
+	    /* If we are moving up one level, remember where we came from, so
 	     * this directory can be highlighted and easily reentered. */
 	    if (strcmp(tail(filelist[selected]), "..") == 0)
-		prev_dir = striponedir(filelist[selected]);
+		present_name = striponedir(filelist[selected]);
 
+	    /* Try opening and reading the selected directory. */
 	    path = mallocstrcpy(path, filelist[selected]);
-
-	    /* Start over again with the new path value. */
-	    goto change_browser_directory;
+	    goto read_directory_contents;
 	} else if (func == do_exit) {
 	    /* Exit from the file browser. */
 	    break;
+#ifndef NANO_TINY
+	} else if (kbinput == KEY_WINCH) {
+	    ;
+#endif
+	} else
+	    unbound_key(kbinput);
+
+#ifndef NANO_TINY
+	/* If the window resized, refresh the file list. */
+	if (kbinput == KEY_WINCH) {
+	    /* Remember the selected file, to be able to reselect it. */
+	    present_name = mallocstrcpy(NULL, filelist[selected]);
+	    /* Reread the contents of the current directory. */
+	    goto read_directory_contents;
 	}
+#endif
     }
+
     titlebar(NULL);
     edit_refresh();
-    if (old_const_update)
-	SET(CONST_UPDATE);
 
     free(path);
-    free(ans);
 
     free_chararray(filelist, filelist_len);
     filelist = NULL;
@@ -366,7 +355,6 @@ char *do_browse_from(const char *inpath)
     struct stat st;
     char *path;
 	/* This holds the tilde-expanded version of inpath. */
-    DIR *dir = NULL;
 
     assert(inpath != NULL);
 
@@ -379,15 +367,21 @@ char *do_browse_from(const char *inpath)
      * at all.  If so, we'll just pass the current directory to
      * do_browser(). */
     if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
-	path = mallocstrassn(path, striponedir(path));
+	path = free_and_assign(path, striponedir(path));
 
 	if (stat(path, &st) == -1 || !S_ISDIR(st.st_mode)) {
+	    char * currentdir = charalloc(PATH_MAX + 1);
+
 	    free(path);
+	    path = getcwd(currentdir, PATH_MAX + 1);
 
-	    path = charalloc(PATH_MAX + 1);
-	    path = getcwd(path, PATH_MAX + 1);
-
-	    if (path != NULL)
+	    if (path == NULL) {
+		free(currentdir);
+		statusline(MILD, "The working directory has disappeared");
+		beep();
+		napms(1200);
+		return NULL;
+	    } else
 		align(&path);
 	}
     }
@@ -399,38 +393,18 @@ char *do_browse_from(const char *inpath)
 	path = mallocstrcpy(path, operating_dir);
 #endif
 
-    if (path != NULL)
-	dir = opendir(path);
-
-    /* If we can't open the path, get out. */
-    if (dir == NULL) {
-	free(path);
-	beep();
-	return NULL;
-    }
-
-    return do_browser(path, dir);
+    return do_browser(path);
 }
 
 /* Set filelist to the list of files contained in the directory path,
  * set filelist_len to the number of files in that list, set longest to
  * the width in columns of the longest filename in that list (between 15
  * and COLS), and set width to the number of files that we can display
- * per line.  longest needs to be at least 15 columns in order to
- * display ".. (parent dir)", as Pico does.  Assume path exists and is a
- * directory. */
-void browser_init(const char *path, DIR *dir)
+ * per line.  And sort the list too. */
+void read_the_list(const char *path, DIR *dir)
 {
     const struct dirent *nextdir;
     size_t i = 0, path_len = strlen(path);
-    int col = 0;
-	/* The maximum number of columns that the filenames will take
-	 * up. */
-    int line = 0;
-	/* The maximum number of lines that the filenames will take
-	 * up. */
-    int filesperline = 0;
-	/* The number of files that we can display per line. */
 
     assert(path != NULL && path[strlen(path) - 1] == '/' && dir != NULL);
 
@@ -450,9 +424,10 @@ void browser_init(const char *path, DIR *dir)
      * in the list whenever possible, as Pico does. */
     longest += 10;
 
-    /* Make sure longest is between 15 and COLS. */
+    /* If needed, make room for ".. (parent dir)". */
     if (longest < 15)
 	longest = 15;
+    /* Make sure we're not wider than the window. */
     if (longest > COLS)
 	longest = COLS;
 
@@ -482,37 +457,15 @@ void browser_init(const char *path, DIR *dir)
      * filelist, so record it. */
     filelist_len = i;
 
-    closedir(dir);
+    assert(filelist != NULL);
 
-    /* Set width to zero, just before we initialize it. */
-    width = 0;
+    /* Sort the list of names. */
+    qsort(filelist, filelist_len, sizeof(char *), diralphasort);
 
-    for (i = 0; i < filelist_len && line < editwinrows; i++) {
-	/* Calculate the number of columns one filename will take up. */
-	col += longest;
-	filesperline++;
-
-	/* Add some space between the columns. */
-	col += 2;
-
-	/* If the next entry isn't going to fit on the current line,
-	 * move to the next line. */
-	if (col > COLS - longest) {
-	    line++;
-	    col = 0;
-
-	    /* If width isn't initialized yet, and we've taken up more
-	     * than one line, it means that width is equal to
-	     * filesperline. */
-	    if (width == 0)
-		width = filesperline;
-	}
-    }
-
-    /* If width isn't initialized yet, and we've taken up only one line,
-     * it means that width is equal to longest. */
-    if (width == 0)
-	width = longest;
+    /* Calculate how many files fit on a line -- feigning room for two
+     * spaces beyond the right edge, and adding two spaces of padding
+     * between columns. */
+    width = (COLS + 2) / (longest + 2);
 }
 
 /* Return the function that is bound to the given key, accepting certain
@@ -551,35 +504,35 @@ void browser_refresh(void)
     size_t i;
     int line = 0, col = 0;
 	/* The current line and column while the list is getting displayed. */
-    char *foo;
+    char *info;
 	/* The additional information that we'll display about a file. */
 
-    titlebar(path_save);
+    titlebar(present_path);
     blank_edit();
 
     wmove(edit, 0, 0);
 
-    i = width * editwinrows * ((selected / width) / editwinrows);
+    i = selected - selected % (editwinrows * width);
 
     for (; i < filelist_len && line < editwinrows; i++) {
 	struct stat st;
-	const char *filetail = tail(filelist[i]);
+	const char *thename = tail(filelist[i]);
 		/* The filename we display, minus the path. */
-	size_t filetaillen = strlenpt(filetail);
+	size_t namelen = strlenpt(thename);
 		/* The length of the filename in columns. */
-	size_t foolen;
+	size_t infolen;
 		/* The length of the file information in columns. */
-	int foomaxlen = 7;
+	int infomaxlen = 7;
 		/* The maximum length of the file information in
 		 * columns: seven for "--", "(dir)", or the file size,
 		 * and 12 for "(parent dir)". */
-	bool dots = (COLS >= 15 && filetaillen >= longest - foomaxlen);
+	bool dots = (COLS >= 15 && namelen >= longest - infomaxlen);
 		/* Do we put an ellipsis before the filename?  Don't set
 		 * this to TRUE if we have fewer than 15 columns (i.e.
 		 * one column for padding, plus seven columns for a
 		 * filename other than ".."). */
-	char *disp = display_string(filetail, dots ? filetaillen -
-		longest + foomaxlen + 4 : 0, longest, FALSE);
+	char *disp = display_string(thename, dots ? namelen -
+		longest + infomaxlen + 4 : 0, longest, FALSE);
 		/* If we put an ellipsis before the filename, reserve
 		 * one column for padding, plus seven columns for "--",
 		 * "(dir)", or the file size, plus three columns for the
@@ -607,25 +560,25 @@ void browser_refresh(void)
 	     * the file browser is open), or it's a symlink that doesn't
 	     * point to a directory, display "--". */
 	    if (stat(filelist[i], &st) == -1 || !S_ISDIR(st.st_mode))
-		foo = mallocstrcpy(NULL, "--");
+		info = mallocstrcpy(NULL, "--");
 	    /* If the file is a symlink that points to a directory,
 	     * display it as a directory. */
 	    else
 		/* TRANSLATORS: Try to keep this at most 7 characters. */
-		foo = mallocstrcpy(NULL, _("(dir)"));
+		info = mallocstrcpy(NULL, _("(dir)"));
 	} else if (S_ISDIR(st.st_mode)) {
 	    /* If the file is a directory, display it as such. */
-	    if (strcmp(filetail, "..") == 0) {
+	    if (strcmp(thename, "..") == 0) {
 		/* TRANSLATORS: Try to keep this at most 12 characters. */
-		foo = mallocstrcpy(NULL, _("(parent dir)"));
-		foomaxlen = 12;
+		info = mallocstrcpy(NULL, _("(parent dir)"));
+		infomaxlen = 12;
 	    } else
-		foo = mallocstrcpy(NULL, _("(dir)"));
+		info = mallocstrcpy(NULL, _("(dir)"));
 	} else {
 	    off_t result = st.st_size;
 	    char modifier;
 
-	    foo = charalloc(foomaxlen + 1);
+	    info = charalloc(infomaxlen + 1);
 
 	    if (st.st_size < (1 << 10))
 		modifier = ' ';  /* bytes */
@@ -643,27 +596,27 @@ void browser_refresh(void)
 	    /* Show the size if less than a terabyte,
 	     * otherwise show "(huge)". */
 	    if (result < (1 << 10))
-		sprintf(foo, "%4ju %cB", (intmax_t)result, modifier);
+		sprintf(info, "%4ju %cB", (intmax_t)result, modifier);
 	    else
 		/* TRANSLATORS: Try to keep this at most 7 characters.
 		 * If necessary, you can leave out the parentheses. */
-		foo = mallocstrcpy(foo, _("(huge)"));
+		info = mallocstrcpy(info, _("(huge)"));
 	}
 
-	/* Make sure foo takes up no more than foomaxlen columns. */
-	foolen = strlenpt(foo);
-	if (foolen > foomaxlen) {
-	    null_at(&foo, actual_x(foo, foomaxlen));
-	    foolen = foomaxlen;
+	/* Make sure info takes up no more than infomaxlen columns. */
+	infolen = strlenpt(info);
+	if (infolen > infomaxlen) {
+	    null_at(&info, actual_x(info, infomaxlen));
+	    infolen = infomaxlen;
 	}
 
-	mvwaddstr(edit, line, col - foolen, foo);
+	mvwaddstr(edit, line, col - infolen, info);
 
 	/* Finish highlighting the currently selected file or directory. */
 	if (i == selected)
 	    wattroff(edit, hilite_attribute);
 
-	free(foo);
+	free(info);
 
 	/* Add some space between the columns. */
 	col += 2;
@@ -693,6 +646,16 @@ void browser_select_dirname(const char *needle)
 	    break;
 	}
     }
+
+    /* If the sought name isn't found, move the highlight so that the
+     * changed selection will be noticed. */
+    if (looking_at == filelist_len) {
+	--selected;
+
+	/* Make sure we stay within the available range. */
+	if (selected >= filelist_len)
+	    selected = filelist_len - 1;
+    }
 }
 
 /* Set up the system variables for a filename search.  Return -1 or -2 if
@@ -704,7 +667,7 @@ int filesearch_init(void)
     int input;
     char *buf;
 
-    if (last_search[0] != '\0') {
+    if (*last_search != '\0') {
 	char *disp = display_string(last_search, 0, COLS / 3, FALSE);
 
 	buf = charalloc(strlen(disp) + 7);
@@ -745,11 +708,8 @@ void findnextfile(const char *needle)
 {
     size_t looking_at = selected;
 	/* The location in the file list of the filename we're looking at. */
-    bool came_full_circle = FALSE;
-	/* Have we reached the starting file again? */
-    const char *filetail = tail(filelist[looking_at]);
-	/* The filename we display, minus the path. */
-    const char *rev_start = filetail, *found = NULL;
+    const char *thename;
+	/* The plain filename, without the path. */
     unsigned stash[sizeof(flags) / sizeof(flags[0])];
 	/* A storage place for the current flag settings. */
 
@@ -764,28 +724,7 @@ void findnextfile(const char *needle)
     /* Step through each filename in the list until a match is found or
      * we've come back to the point where we started. */
     while (TRUE) {
-	found = strstrwrapper(filetail, needle, rev_start);
-
-	/* If we've found a match and it's not the same filename where
-	 * we started, then we're done. */
-	if (found != NULL && looking_at != selected)
-	    break;
-
-	/* If we've found a match and we're back at the beginning, then
-	 * it's the only occurrence. */
-	if (found != NULL && came_full_circle) {
-	    statusbar(_("This is the only occurrence"));
-	    break;
-	}
-
-	if (came_full_circle) {
-	    /* We're back at the beginning and didn't find anything. */
-	    not_found_msg(needle);
-	    return;
-	}
-
-	/* Move to the next filename in the list.  If we've reached the
-	 * end of the list, wrap around. */
+	/* Move to the next filename in the list, or back to the first. */
 	if (looking_at < filelist_len - 1)
 	    looking_at++;
 	else {
@@ -793,13 +732,22 @@ void findnextfile(const char *needle)
 	    statusbar(_("Search Wrapped"));
 	}
 
-	if (looking_at == selected)
-	    /* We've reached the original starting file. */
-	    came_full_circle = TRUE;
+	/* Get the bare filename, without the path. */
+	thename = tail(filelist[looking_at]);
 
-	filetail = tail(filelist[looking_at]);
+	/* If the needle matches, we're done.  And if we're back at the file
+	 * where we started, it is the only occurrence. */
+	if (strstrwrapper(thename, needle, thename)) {
+	    if (looking_at == selected)
+		statusbar(_("This is the only occurrence"));
+	    break;
+	}
 
-	rev_start = filetail;
+	/* If we're back at the beginning and didn't find any match... */
+	if (looking_at == selected) {
+	    not_found_msg(needle);
+	    break;
+	}
     }
 
     /* Restore the settings of all flags. */
@@ -812,11 +760,9 @@ void findnextfile(const char *needle)
 /* Search for a filename. */
 void do_filesearch(void)
 {
-    if (filesearch_init() != 0) {
-	/* Cancelled, or a blank search string, or done something. */
-	bottombars(MBROWSER);
+    /* If the user cancelled or jumped to first or last file, don't search. */
+    if (filesearch_init() != 0)
 	return;
-    }
 
     /* If answer is now "", copy last_search into answer. */
     if (*answer == '\0')
@@ -825,21 +771,18 @@ void do_filesearch(void)
 	last_search = mallocstrcpy(last_search, answer);
 
 #ifndef DISABLE_HISTORIES
-    /* If answer is not "", add this search string to the search history
-     * list. */
-    if (answer[0] != '\0')
+    /* If answer is not empty, add the string to the search history list. */
+    if (*answer != '\0')
 	update_history(&search_history, answer);
 #endif
 
     findnextfile(answer);
-
-    bottombars(MBROWSER);
 }
 
-/* Search for the last given filename again without prompting. */
+/* Search again for the last given filename, without prompting. */
 void do_fileresearch(void)
 {
-    if (last_search[0] == '\0')
+    if (*last_search == '\0')
 	statusbar(_("No current search pattern"));
     else
 	findnextfile(last_search);
