@@ -21,6 +21,7 @@
  **************************************************************************/
 
 #include "proto.h"
+#include "revision.h"
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -602,49 +603,46 @@ void die(const char *msg, ...)
     vfprintf(stderr, msg, ap);
     va_end(ap);
 
-    /* Save the current file buffer if it's been modified. */
+#ifndef NANO_TINY
+    /* If the current buffer has a lockfile, remove it. */
+    if (ISSET(LOCKING) && openfile->lock_filename)
+	delete_lockfile(openfile->lock_filename);
+#endif
+
+    /* If the current file buffer was modified, save it. */
     if (openfile && openfile->modified) {
-	/* If we've partitioned the filestruct, unpartition it now. */
+	/* If the filestruct is partitioned, unpartition it first. */
 	if (filepart != NULL)
 	    unpartition_filestruct(&filepart);
 
-	die_save_file(openfile->filename
-#ifndef NANO_TINY
-		, openfile->current_stat
-#endif
-		);
+	die_save_file(openfile->filename, openfile->current_stat);
     }
 
 #ifndef DISABLE_MULTIBUFFER
     /* Save all of the other modified file buffers, if any. */
     if (openfile != NULL) {
-	openfilestruct *tmp = openfile;
+	openfilestruct *firstone = openfile;
 
-	while (tmp != openfile->next) {
+	while (openfile->next != firstone) {
 	    openfile = openfile->next;
 
-	    /* Save the current file buffer if it's been modified. */
-	    if (openfile->modified)
-		die_save_file(openfile->filename
 #ifndef NANO_TINY
-			, openfile->current_stat
+	    if (ISSET(LOCKING) && openfile->lock_filename)
+		delete_lockfile(openfile->lock_filename);
 #endif
-			);
+	    if (openfile->modified)
+		die_save_file(openfile->filename, openfile->current_stat);
 	}
     }
 #endif
 
-    /* Get out. */
+    /* Abandon the building. */
     exit(1);
 }
 
 /* Save the current file under the name specified in die_filename, which
  * is modified to be unique if necessary. */
-void die_save_file(const char *die_filename
-#ifndef NANO_TINY
-	, struct stat *die_stat
-#endif
-	)
+void die_save_file(const char *die_filename, struct stat *die_stat)
 {
     char *targetname;
     bool failed = TRUE;
@@ -688,11 +686,14 @@ void die_save_file(const char *die_filename
     free(targetname);
 }
 
+#define TOP_ROWS    (ISSET(MORE_SPACE) ? 1 : 2)
+#define BOTTOM_ROWS    (ISSET(NO_HELP) ? 1 : 3)
+
 /* Initialize the three window portions nano uses. */
 void window_init(void)
 {
     /* If the screen height is too small, get out. */
-    editwinrows = LINES - 5 + more_space() + no_help();
+    editwinrows = LINES - TOP_ROWS - BOTTOM_ROWS;
     if (COLS < MIN_EDITOR_COLS || editwinrows < MIN_EDITOR_ROWS)
 	die(_("Window size is too small for nano...\n"));
 
@@ -713,10 +714,9 @@ void window_init(void)
 	delwin(bottomwin);
 
     /* Set up the windows. */
-    topwin = newwin(2 - more_space(), COLS, 0, 0);
-    edit = newwin(editwinrows, COLS, 2 - more_space(), 0);
-    bottomwin = newwin(3 - no_help(), COLS, editwinrows + (2 -
-	more_space()), 0);
+    topwin = newwin(TOP_ROWS, COLS, 0, 0);
+    edit = newwin(editwinrows, COLS, TOP_ROWS, 0);
+    bottomwin = newwin(BOTTOM_ROWS, COLS, TOP_ROWS + editwinrows, 0);
 
     /* Turn the keypad on for the windows, if necessary. */
     if (!ISSET(REBIND_KEYPAD)) {
@@ -859,6 +859,8 @@ void usage(void)
 #ifndef NANO_TINY
     print_opt("-W", "--wordbounds",
 	N_("Detect word boundaries more accurately"));
+    print_opt(_("-X <str>"), _("--wordchars=<str>"),
+	N_("Which other characters are word parts"));
 #endif
 #ifndef DISABLE_COLOR
     if (!ISSET(RESTRICTED))
@@ -918,7 +920,7 @@ void usage(void)
 void version(void)
 {
 #ifdef REVISION
-    printf(" nano from git, commit %s (after %s)\n", REVISION, VERSION);
+    printf(" nano from git, %s\n", REVISION);
 #else
     printf(_(" nano, version %s\n"), VERSION);
 #endif
@@ -1040,21 +1042,6 @@ void version(void)
     printf("\n");
 }
 
-/* Return 1 if the MORE_SPACE flag is set, and 0 otherwise.  This is
- * used to calculate the sizes and Y coordinates of the subwindows. */
-int more_space(void)
-{
-    return ISSET(MORE_SPACE) ? 1 : 0;
-}
-
-/* Return 2 if the NO_HELP flag is set, and 0 otherwise.  This is used
- * to calculate the sizes and Y coordinates of the subwindows, because
- * having NO_HELP adds two lines to the edit window. */
-int no_help(void)
-{
-    return ISSET(NO_HELP) ? 2 : 0;
-}
-
 /* Indicate that the current file has no name, in a way that gets the
  * user's attention.  This is used when trying to save a file with no
  * name with the TEMP_FILE flag set, just before the filename prompt. */
@@ -1297,10 +1284,9 @@ RETSIGTYPE do_continue(int signal)
 #endif
 
 #ifndef NANO_TINY
-    /* Perhaps the user resized the window while we slept.  Handle it,
-     * and restore the terminal to its previous state and update the
-     * screen in the process. */
-    handle_sigwinch(0);
+    /* Perhaps the user resized the window while we slept.  So act as if,
+     * and restore the terminal to its previous state in the process. */
+    regenerate_screen();
 #else
     /* Restore the terminal to its previous state. */
     terminal_init();
@@ -1550,12 +1536,13 @@ void terminal_init(void)
 /* Say that an unbound key was struck, and if possible which one. */
 void unbound_key(int code)
 {
-    if (func_key)
+    if (!is_byte(code))
 	statusline(ALERT, _("Unbound key"));
     else if (meta_key) {
-	if (0x60 < code && code < 0x7B)
-	    code -= 0x20;
-	statusline(ALERT, _("Unbound key: M-%c"), code);
+	if (code == '[')
+	    statusline(ALERT, _("Unbindable key: M-["));
+	else
+	    statusline(ALERT, _("Unbound key: M-%c"), toupper(code));
     } else if (code < 0x20)
 	statusline(ALERT, _("Unbound key: ^%c"), code + 0x40);
     else
@@ -1569,7 +1556,7 @@ int do_input(bool allow_funcs)
 {
     int input;
 	/* The keystroke we read in: a character or a shortcut. */
-    static int *puddle = NULL;
+    static char *puddle = NULL;
 	/* The input buffer for actual characters. */
     static size_t depth = 0;
 	/* The length of the input buffer. */
@@ -1587,7 +1574,7 @@ int do_input(bool allow_funcs)
 #endif
 
 #ifndef DISABLE_MOUSE
-    if (func_key && input == KEY_MOUSE) {
+    if (input == KEY_MOUSE) {
 	/* We received a mouse click. */
 	if (do_mouse() == 1)
 	    /* The click was on a shortcut -- read in the character
@@ -1609,10 +1596,8 @@ int do_input(bool allow_funcs)
     /* If we got a non-high-bit control key, a meta key sequence, or a
      * function key, and it's not a shortcut or toggle, throw it out. */
     if (!have_shortcut) {
-	if (is_ascii_cntrl_char(input) || meta_key || func_key) {
+	if (is_ascii_cntrl_char(input) || meta_key || !is_byte(input)) {
 	    unbound_key(input);
-	    meta_key = FALSE;
-	    func_key = FALSE;
 	    input = ERR;
 	}
     }
@@ -1625,9 +1610,9 @@ int do_input(bool allow_funcs)
 	    if (ISSET(VIEW_MODE))
 		print_view_warning();
 	    else {
-		depth++;
-		puddle = (int *)nrealloc(puddle, depth * sizeof(int));
-		puddle[depth - 1] = input;
+		/* Store the byte, and leave room for a terminating zero. */
+		puddle = charealloc(puddle, depth + 2);
+		puddle[depth++] = (char)input;
 	    }
 	}
 
@@ -1644,18 +1629,10 @@ int do_input(bool allow_funcs)
 #endif
 
 	    if (puddle != NULL) {
-		/* Display all the characters in the input buffer at
-		 * once, filtering out control characters. */
-		char *output = charalloc(depth + 1);
-		size_t i;
-
-		for (i = 0; i < depth; i++)
-		    output[i] = (char)puddle[i];
-		output[i] = '\0';
-
-		do_output(output, depth, FALSE);
-
-		free(output);
+		/* Insert all bytes in the input buffer into the edit buffer
+		 * at once, filtering out any low control codes. */
+		puddle[depth] = '\0';
+		do_output(puddle, depth, FALSE);
 
 		/* Empty the input buffer. */
 		free(puddle);
@@ -1841,8 +1818,8 @@ void do_output(char *output, size_t output_len, bool allow_cntrls)
 #endif
 
     while (i < output_len) {
-	/* If control codes are allowed, encode a null as a newline, and
-	 * let a newline character create a whole new line. */
+	/* If control codes are allowed, encode a verbatim null as a newline,
+	 * and let a verbatim ^J create a whole new line. */
 	if (allow_cntrls) {
 	    if (output[i] == '\0')
 		output[i] = '\n';
@@ -2003,6 +1980,7 @@ int main(int argc, char **argv)
 	{"smooth", 0, NULL, 'S'},
 	{"quickblank", 0, NULL, 'U'},
 	{"wordbounds", 0, NULL, 'W'},
+	{"wordchars", 1, NULL, 'X'},
 	{"autoindent", 0, NULL, 'i'},
 	{"cut", 0, NULL, 'k'},
 	{"unix", 0, NULL, 'u'},
@@ -2048,11 +2026,11 @@ int main(int argc, char **argv)
     while ((optchr =
 #ifdef HAVE_GETOPT_LONG
 	getopt_long(argc, argv,
-		"ABC:DEFGHIKLNOPQ:RST:UVWY:abcdefghijklmno:pqr:s:tuvwxz$",
+		"ABC:DEFGHIKLNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxz$",
 		long_options, NULL)
 #else
 	getopt(argc, argv,
-		"ABC:DEFGHIKLNOPQ:RST:UVWY:abcdefghijklmno:pqr:s:tuvwxz$")
+		"ABC:DEFGHIKLNOPQ:RST:UVWX:Y:abcdefghijklmno:pqr:s:tuvwxz$")
 #endif
 		) != -1) {
 	switch (optchr) {
@@ -2154,6 +2132,9 @@ int main(int argc, char **argv)
 	    case 'W':
 		SET(WORD_BOUNDS);
 		break;
+	    case 'X':
+		word_chars = mallocstrcpy(word_chars, optarg);
+		break;
 #endif
 #ifndef DISABLE_COLOR
 	    case 'Y':
@@ -2287,6 +2268,7 @@ int main(int argc, char **argv)
 #endif
 #ifndef NANO_TINY
 	char *backup_dir_cpy = backup_dir;
+	char *word_chars_cpy = word_chars;
 #endif
 #ifndef DISABLE_JUSTIFY
 	char *quotestr_cpy = quotestr;
@@ -2305,6 +2287,7 @@ int main(int argc, char **argv)
 #endif
 #ifndef NANO_TINY
 	backup_dir = NULL;
+	word_chars = NULL;
 #endif
 #ifndef DISABLE_JUSTIFY
 	quotestr = NULL;
@@ -2335,6 +2318,10 @@ int main(int argc, char **argv)
 	    free(backup_dir);
 	    backup_dir = backup_dir_cpy;
 	}
+	if (word_chars_cpy != NULL) {
+	    free(word_chars);
+	    word_chars = word_chars_cpy;
+	}
 #endif
 #ifndef DISABLE_JUSTIFY
 	if (quotestr_cpy != NULL) {
@@ -2495,6 +2482,9 @@ int main(int argc, char **argv)
     /* Set up the terminal state. */
     terminal_init();
 
+    /* Check whether we're running on a Linux console. */
+    console = (getenv("DISPLAY") == NULL);
+
 #ifdef DEBUG
     fprintf(stderr, "Main: set up windows\n");
 #endif
@@ -2514,25 +2504,32 @@ int main(int argc, char **argv)
 #ifndef DISABLE_COLOR
     set_colorpairs();
 #else
-    interface_color_pair[TITLE_BAR].pairnum = hilite_attribute;
-    interface_color_pair[STATUS_BAR].pairnum = hilite_attribute;
-    interface_color_pair[KEY_COMBO].pairnum = hilite_attribute;
-    interface_color_pair[FUNCTION_TAG].pairnum = A_NORMAL;
-    interface_color_pair[TITLE_BAR].bright = FALSE;
-    interface_color_pair[STATUS_BAR].bright = FALSE;
-    interface_color_pair[KEY_COMBO].bright = FALSE;
-    interface_color_pair[FUNCTION_TAG].bright = FALSE;
+    interface_color_pair[TITLE_BAR] = hilite_attribute;
+    interface_color_pair[STATUS_BAR] = hilite_attribute;
+    interface_color_pair[KEY_COMBO] = hilite_attribute;
+    interface_color_pair[FUNCTION_TAG] = A_NORMAL;
 #endif
 
 #if !defined(NANO_TINY) && defined(HAVE_KEY_DEFINED)
     const char *keyvalue;
-    /* Ask ncurses for the key codes for Control+Left and Control+Right. */
+    /* Ask ncurses for the key codes for Control+Left/Right/Up/Down. */
     keyvalue = tigetstr("kLFT5");
     if (keyvalue != 0 && keyvalue != (char *)-1)
 	controlleft = key_defined(keyvalue);
     keyvalue = tigetstr("kRIT5");
     if (keyvalue != 0 && keyvalue != (char *)-1)
 	controlright = key_defined(keyvalue);
+    keyvalue = tigetstr("kUP5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controlup = key_defined(keyvalue);
+    keyvalue = tigetstr("kDN5");
+    if (keyvalue != 0 && keyvalue != (char *)-1)
+	controldown = key_defined(keyvalue);
+#endif
+
+#ifndef USE_SLANG
+    /* Tell ncurses to pass the Esc key quickly. */
+    set_escdelay(50);
 #endif
 
 #ifdef DEBUG
