@@ -58,7 +58,7 @@
 
 /* Macros for flags, indexing each bit in a small array. */
 #define FLAGS(flag) flags[((flag) / (sizeof(unsigned) * 8))]
-#define FLAGMASK(flag) (1 << ((flag) % (sizeof(unsigned) * 8)))
+#define FLAGMASK(flag) ((unsigned)1 << ((flag) % (sizeof(unsigned) * 8)))
 #define SET(flag) FLAGS(flag) |= FLAGMASK(flag)
 #define UNSET(flag) FLAGS(flag) &= ~FLAGMASK(flag)
 #define ISSET(flag) ((FLAGS(flag) & FLAGMASK(flag)) != 0)
@@ -92,6 +92,10 @@
 #include <curses.h>
 #endif /* CURSES_H */
 
+#if defined(NCURSES_VERSION_MAJOR) && (NCURSES_VERSION_MAJOR < 6)
+#define USING_OLD_NCURSES yes
+#endif
+
 #ifdef ENABLE_NLS
 /* Native language support. */
 #ifdef HAVE_LIBINTL_H
@@ -173,7 +177,7 @@ typedef enum {
 
 typedef enum {
     OVERWRITE, APPEND, PREPEND
-} append_type;
+} kind_of_writing_type;
 
 typedef enum {
     UPWARD, DOWNWARD
@@ -183,10 +187,6 @@ typedef enum {
     CENTERING, FLOWING, STATIONARY
 } update_type;
 
-typedef enum {
-    CONTROL, META, FKEY, RAWINPUT
-}  key_type;
-
 typedef enum {
     ADD, DEL, BACK, CUT, CUT_EOF, REPLACE,
 #ifndef DISABLE_WRAPPING
@@ -199,14 +199,6 @@ typedef enum {
 } undo_type;
 
 /* Structure types. */
-typedef struct color_pair {
-    int pairnum;
-	/* The color pair number used for this foreground color and
-	 * background color. */
-    bool bright;
-	/* Is this color A_BOLD? */
-} color_pair;
-
 #ifndef DISABLE_COLOR
 typedef struct colortype {
     short fg;
@@ -218,6 +210,8 @@ typedef struct colortype {
     int pairnum;
 	/* The color pair number used for this foreground color and
 	 * background color. */
+    int attributes;
+	/* Pair number and brightness composed into ready-to-use attributes. */
     int rex_flags;
 	/* The regex compilation flags (with or without REG_ICASE). */
     char *start_regex;
@@ -399,6 +393,8 @@ typedef struct openfilestruct {
 	/* The file's y-coordinate position. */
     bool modified;
 	/* Whether the file has been modified. */
+    struct stat *current_stat;
+	/* The file's current stat information. */
 #ifndef NANO_TINY
     bool mark_set;
 	/* Whether the mark is on in this file. */
@@ -408,8 +404,6 @@ typedef struct openfilestruct {
 	/* The file's mark's x-coordinate position, if any. */
     file_format fmt;
 	/* The file's format. */
-    struct stat *current_stat;
-	/* The file's current stat information. */
     undo *undotop;
 	/* The top of the undo list. */
     undo *current_undo;
@@ -441,12 +435,12 @@ typedef struct rcoption {
 #endif
 
 typedef struct sc {
-    char *keystr;
-	/* The shortcut key for a function, ASCII version. */
-    key_type type;
-	/* What kind of command key it is, for convenience later. */
-    int seq;
-	/* The actual sequence to check once the type is determined. */
+    const char *keystr;
+	/* The string that describes a keystroke, like "^C" or "M-R". */
+    bool meta;
+	/* Whether this is a Meta keystroke. */
+    int keycode;
+	/* The integer that, together with meta, identifies the keystroke. */
     int menus;
 	/* Which menus this applies to. */
     void (*scfunc)(void);
@@ -556,16 +550,16 @@ enum
 #define MMOST  (MMAIN|MWHEREIS|MREPLACE|MREPLACEWITH|MGOTOLINE|MWRITEFILE|MINSERTFILE|\
 		MEXTCMD|MBROWSER|MWHEREISFILE|MGOTODIR|MSPELL|MLINTER)
 
-/* Control key sequences.  Changing these would be very, very bad. */
-#define NANO_CONTROL_SPACE 0
-#define NANO_CONTROL_I 9
-#define NANO_CONTROL_3 27
-#define NANO_CONTROL_7 31
-#define NANO_CONTROL_8 127
+/* Basic control codes. */
+#define TAB_CODE  0x09
+#define ESC_CODE  0x1B
+#define DEL_CODE  0x7F
 
 /* Codes for "modified" Arrow keys, beyond KEY_MAX of ncurses. */
 #define CONTROL_LEFT 0x401
 #define CONTROL_RIGHT 0x402
+#define CONTROL_UP 0x403
+#define CONTROL_DOWN 0x404
 
 #ifndef NANO_TINY
 /* An imaginary key for when we get a SIGWINCH (window resize). */
@@ -604,10 +598,4 @@ enum
 /* The largest size_t number that doesn't have the high bit set. */
 #define HIGHEST_POSITIVE ((~(size_t)0) >> 1)
 
-#ifdef REVISION
-#define BRANDING PACKAGE_VERSION"-git  "REVISION
-#else
-#define BRANDING PACKAGE_STRING
-#endif
-
 #endif /* !NANO_H */
