@@ -77,35 +77,33 @@ bool parse_num(const char *str, ssize_t *val)
     return TRUE;
 }
 
-/* Read two ssize_t's, separated by a comma, from str, and store them in
- * *line and *column (if they're not both NULL).  Return FALSE on error,
- * or TRUE otherwise. */
+/* Read two numbers, separated by a comma, from str, and store them in
+ * *line and *column.  Return FALSE on error, and TRUE otherwise. */
 bool parse_line_column(const char *str, ssize_t *line, ssize_t *column)
 {
-    bool retval = TRUE;
+    bool retval;
+    char *firstpart;
     const char *comma;
 
-    assert(str != NULL);
+    while (*str == ' ')
+       str++;
 
-    comma = strchr(str, ',');
+    comma = strpbrk(str, "m,. /;");
 
-    if (comma != NULL && column != NULL) {
-	if (!parse_num(comma + 1, column))
-	    retval = FALSE;
-    }
+    if (comma == NULL)
+	return parse_num(str, line);
 
-    if (line != NULL) {
-	if (comma != NULL) {
-	    char *str_line = mallocstrncpy(NULL, str, comma - str + 1);
-	    str_line[comma - str] = '\0';
+    if (!parse_num(comma + 1, column))
+	return FALSE;
 
-	    if (str_line[0] != '\0' && !parse_num(str_line, line))
-		retval = FALSE;
+    if (comma == str)
+	return TRUE;
 
-	    free(str_line);
-	} else if (!parse_num(str, line))
-	    retval = FALSE;
-    }
+    firstpart = mallocstrcpy(NULL, str);
+    firstpart[comma - str] = '\0';
+
+    retval = parse_num(firstpart, line);
+    free(firstpart);
 
     return retval;
 }
@@ -292,12 +290,11 @@ bool is_separate_word(size_t position, size_t length, const char *buf)
     parse_mbchar(buf + move_mbleft(buf, position), before, NULL);
     parse_mbchar(buf + word_end, after, NULL);
 
-    /* If we're at the beginning of the line or the character before the
-     * word isn't a non-punctuation "word" character, and if we're at
-     * the end of the line or the character after the word isn't a
-     * non-punctuation "word" character, we have a whole word. */
-    retval = (position == 0 || !is_word_mbchar(before, FALSE)) &&
-		(word_end == strlen(buf) || !is_word_mbchar(after, FALSE));
+    /* If the word starts at the beginning of the line OR the character before
+     * the word isn't a letter, and if the word ends at the end of the line OR
+     * the character after the word isn't a letter, we have a whole word. */
+    retval = (position == 0 || !is_alpha_mbchar(before)) &&
+		(word_end == strlen(buf) || !is_alpha_mbchar(after));
 
     free(before);
     free(after);
@@ -406,8 +403,8 @@ void *nrealloc(void *ptr, size_t howmuch)
     return r;
 }
 
-/* Copy the first n characters of one malloc()ed string to another
- * pointer.  Should be used as: "dest = mallocstrncpy(dest, src, n);". */
+/* Allocate and copy the first n characters of the given src string, after
+ * freeing the destination.  Usage: "dest = mallocstrncpy(dest, src, n);". */
 char *mallocstrncpy(char *dest, const char *src, size_t n)
 {
     if (src == NULL)
@@ -422,17 +419,15 @@ char *mallocstrncpy(char *dest, const char *src, size_t n)
     return dest;
 }
 
-/* Copy one malloc()ed string to another pointer.  Should be used as:
+/* Free the dest string and return a malloc'ed copy of src.  Should be used as:
  * "dest = mallocstrcpy(dest, src);". */
 char *mallocstrcpy(char *dest, const char *src)
 {
     return mallocstrncpy(dest, src, (src == NULL) ? 1 : strlen(src) + 1);
 }
 
-/* Free the malloc()ed string at dest and return the malloc()ed string
- * at src.  Should be used as: "answer = mallocstrassn(answer,
- * real_dir_from_tilde(answer));". */
-char *mallocstrassn(char *dest, char *src)
+/* Free the string at dest and return the string at src. */
+char *free_and_assign(char *dest, char *src)
 {
     free(dest);
     return src;
